/*
 * data_handling.c
 *
 * Created: 4/20/2012 12:02:44 PM
 *  Author: Doug
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include "dac_driver.h"
#include "dma_driver.h"
#include "data_handling.h"
#include "wifly.h"
#include "board.h"
#include "usart_dma_test.h"
#include "Final_Project.h"

#ifdef STEREO
#define AUDIO_BUFFER_SIZE 4096
#else
#define AUDIO_BUFFER_SIZE 8192
#endif //STEREO

#ifdef STEREO
#define AUDIO_MASK 0xfff
#else
#define AUDIO_MASK 0x1fff
#endif //STEREO

#define DMA_CHANNEL_RIGHT  &DMA.CH3
#define DMA_CHANNEL_LEFT  &DMA.CH2

#ifdef STEREO
uint8_t right_audio[AUDIO_BUFFER_SIZE];
uint8_t left_audio[AUDIO_BUFFER_SIZE];
#else
uint8_t left_audio[AUDIO_BUFFER_SIZE];
uint8_t right_audio[10];
#endif //STEREO

uint8_t dma_rx_buffer0[DMA_RX_BUFFER_LEN];
uint8_t dma_rx_buffer1[DMA_RX_BUFFER_LEN];
uint8_t dma_rx_buffer2[DMA_RX_BUFFER_LEN];

// writes must be surrounded with cli(), sei()
volatile uint16_t right_audio_head = 0;
volatile uint16_t left_audio_head = 0;
volatile uint16_t right_audio_tail = 0;
volatile uint16_t left_audio_tail = 0;

volatile uint8_t dma_rx_buffer_number;
volatile uint8_t dma_rx_buffer_ready = false;

//Flag to tell Wifly to send packet
extern volatile uint8_t send_packet;

//Number of overflows = number of samples handled by DAC since last DMA setup
extern volatile uint8_t num_overflows_since_starting_DMA;

extern volatile uint16_t timer1;
extern volatile uint16_t timer2;
extern volatile uint8_t num_dma_transfer;
extern volatile uint8_t first_time;

inline uint16_t bytes_in_left_audio_buffer(void)
{
	//data goes in the head and is popped from the tail.
	if (left_audio_head >= left_audio_tail) return (left_audio_head - left_audio_tail);	
	else return (left_audio_head + AUDIO_BUFFER_SIZE - left_audio_tail);
}

// copies data from a buffer to the audio buffers
// input sample width is 8 bits with 1 channel
// output sample width is 16 bits written to both channels
inline int8_t copy_8bit_mono_to_buffer_no_zero(uint8_t * data, uint16_t len)
{ 
	for (uint16_t i = 0; i < len; i++)
	{
		if (bytes_in_left_audio_buffer() >= (AUDIO_BUFFER_SIZE - 1) ) {
			return -1;
		}
		//send same data to both channels
		left_audio[left_audio_head] = data[i];
		cli();
		left_audio_head += 1;
		left_audio_head &= AUDIO_MASK;	// wrap around head when it gets too big
		sei();
		
// 		right_audio[right_audio_head] = data[i];
// 		cli();
// 		right_audio_head +=1;
// 		right_audio_head &= AUDIO_MASK;// wrap around head when it gets too big
// 		sei();
	}
	return 0;
}

// copies data from a buffer to the audio buffers
// input sample width is 8 bits with 1 channel
// output sample width is 16 bits written to both channels
inline int8_t copy_8bit_stereo_to_buffer_no_zero(uint8_t * data, uint16_t len)
{ 
	for (uint16_t i = 0; i < len; i += 2)
	{
		if (bytes_in_left_audio_buffer() >= (AUDIO_BUFFER_SIZE - 1) ) {
			return -1;
		}
		//send same data to both channels
		left_audio[left_audio_head] = data[i];
		cli();
		left_audio_head += 1;
		left_audio_head &= AUDIO_MASK;	// wrap around head when it gets too big
		sei();
		
		right_audio[right_audio_head] = data[i+1];
		cli();
		right_audio_head +=1;
		right_audio_head &= AUDIO_MASK;// wrap around head when it gets too big
		sei();
	}
	return 0;
}


//send data to DMA
inline void send_data_to_DMA_no_zero(void){
	static uint8_t need_update = 0;
	if (need_update == 1){
		//increment tails by number of bytes sent via DMA 

		left_audio_tail += (DMA_NUM_BURSTS_IN_BLOCK );  
		left_audio_tail &= AUDIO_MASK; //when head equals 2048 set it back to 0
		#ifdef STEREO
		right_audio_tail += (DMA_NUM_BURSTS_IN_BLOCK ); 
		right_audio_tail &= AUDIO_MASK; //when head equals 2048 set it back to 0
		#endif //STEREO
		need_update = 0;
	}
	
	//see if there is more data in the channel
	if (bytes_in_left_audio_buffer() >= (DMA_NUM_BURSTS_IN_BLOCK ))
	{
		// wait til the DMA channels are free
		num_overflows_since_starting_DMA = 0;
		if (!first_time){
	 		while ( !DMA_ReturnStatus_non_blocking( DMA_CHANNEL_LEFT ))
	 		{
	 			nop();
	 		}
			//clear flags by writing to them
			DMA.CH2.CTRLB = (DMA_CH_ERRIF_bm | DMA_CH_TRNIF_bm);
		} else {
			first_time = 0;	
		}
		
		#ifdef STEREO
		dma_setup_right_no_zero(&right_audio[right_audio_tail], DMA_NUM_BURSTS_IN_BLOCK);
		#else
		dma_setup_right_no_zero(&left_audio[left_audio_tail], DMA_NUM_BURSTS_IN_BLOCK);
		#endif //STEREO
		dma_setup_left_no_zero(&left_audio[left_audio_tail], DMA_NUM_BURSTS_IN_BLOCK);
		need_update = 1;
		timer2 = 0;
	} else {
		GREEN_TOGGLE();
		disable_audio();
	}
}

//increase the number of overflows
ISR(TCC0_OVF_vect)
{
	if (num_overflows_since_starting_DMA <= DMA_NUM_BURSTS_IN_BLOCK)
	{
		num_overflows_since_starting_DMA++;
	}
	
	//when there have been enough overflows tell the DMA to send more data to the DAC
	if (num_overflows_since_starting_DMA >= DMA_NUM_BURSTS_IN_BLOCK)
	{
		//num_overflows_since_starting_DMA = 0;
		//this overflow initiated the last transfer 
		//send data to DMA because DAC needs more data                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  

		send_data_to_DMA_no_zero();
	}	
}


//Do nothing in this interrupt handler
ISR(TCD0_OVF_vect){
	cli();
	timer1++;
	timer2++;
	sei();
}
		
inline void DMA_Setup_DACB0_no_zero( DMA_CH_t * dmaChannel,
                const void * src,
                void * dest,
                int16_t blockSize )
{
    DMA_Enable();

	DMA_DisableChannel(dmaChannel);
	DMA_ResetChannel(dmaChannel);
	DMA_SetupBlock(
                    dmaChannel,
                    src,
                    DMA_CH_SRCRELOAD_BLOCK_gc, 
                    DMA_CH_SRCDIR_INC_gc,
                    dest,
                    DMA_CH_DESTRELOAD_BURST_gc,
                    DMA_CH_DESTDIR_INC_gc,
                    blockSize,
                    DMA_CH_BURSTLEN_1BYTE_gc,
                    1,
                    false
                   );

    // DACB0 DRE will trigger DMA
    DMA_SetTriggerSource( dmaChannel, DMA_CH_TRIGSRC_DACB_CH0_gc );
    DMA_EnableSingleShot( dmaChannel );
}

inline void DMA_Setup_DACB1_no_zero( DMA_CH_t * dmaChannel,
                const void * src,
                void * dest,
                int16_t blockSize )
{
    DMA_Enable();
	
	DMA_DisableChannel(dmaChannel);
	DMA_ResetChannel(dmaChannel);
	
	DMA_SetupBlock(
                    dmaChannel,
                    src,
                    DMA_CH_SRCRELOAD_BLOCK_gc, 
                    DMA_CH_SRCDIR_INC_gc,
                    dest,
                    DMA_CH_DESTRELOAD_BURST_gc,
                    DMA_CH_DESTDIR_INC_gc,
                    blockSize,
                    DMA_CH_BURSTLEN_1BYTE_gc,
                    1,		
                    false
                   );

    // DACB0 DRE will trigger DMA
    DMA_SetTriggerSource( dmaChannel, DMA_CH_TRIGSRC_DACB_CH1_gc );
    DMA_EnableSingleShot( dmaChannel );
}

inline void dma_setup_left_no_zero( const void * src, int16_t blockSize )
{
    DMA_Setup_DACB0_no_zero(DMA_CHANNEL_LEFT, src,  (void *) &DACB.CH0DATAH, blockSize);

    DMA_EnableChannel( DMA_CHANNEL_LEFT );
}

inline void dma_setup_right_no_zero( const void * src, int16_t blockSize)
{
    DMA_Setup_DACB1_no_zero(DMA_CHANNEL_RIGHT, src,  (void *) &DACB.CH1DATAH, blockSize);

    DMA_EnableChannel( DMA_CHANNEL_RIGHT );
}

void SetupReceiveChannel( void )
{
    DisableWiflyUsartRxISR();
	
	DMA_Enable();

	DMA_SetupBlock( 
                    DMA_RX_Channel,
                    (void *) &USART_WIFLY.DATA,
                    DMA_CH_SRCRELOAD_NONE_gc, 
                    DMA_CH_SRCDIR_FIXED_gc,
                    dma_rx_buffer0,
                    DMA_CH_DESTRELOAD_NONE_gc, 
                    DMA_CH_DESTDIR_INC_gc,
                    DMA_RX_BUFFER_LEN, 
                    DMA_CH_BURSTLEN_1BYTE_gc, 
                    0, // Perform once
                    false
                   );
    
    DMA_EnableSingleShot( DMA_RX_Channel );
    // USART Trigger source, Receive complete
    DMA_SetTriggerSource( DMA_RX_Channel, DMA_CH_TRIGSRC_USART_RXC_gc); 
	
	DMA_SetIntLevel( DMA_RX_Channel, DMA_CH_TRNINTLVL_HI_gc, DMA_CH_ERRINTLVL_HI_gc );
    PMIC.CTRL |= PMIC_HILVLEN_bm;
	
	DMA_EnableChannel( DMA_RX_Channel );
	dma_rx_buffer_number = 0;
}

// DMA CH0 Interrupt service routine. Clear interrupt flags after check
ISR(DMA_CH1_vect)
{	
	if (DMA.INTFLAGS & DMA_CH1TRNIF_bm) 
    {
        DMA.INTFLAGS = DMA_CH1TRNIF_bm;
		DMA_DisableChannel( DMA_RX_Channel );
		DMA_ResetChannel(DMA_RX_Channel);
		if (dma_rx_buffer_number == 0)
		{
			dma_rx_buffer_number = 1;
			DMA_SetupBlock( 
                DMA_RX_Channel,
                (void *) &USART_WIFLY.DATA,
                DMA_CH_SRCRELOAD_NONE_gc, 
                DMA_CH_SRCDIR_FIXED_gc,
                dma_rx_buffer1,
                DMA_CH_DESTRELOAD_NONE_gc, 
                DMA_CH_DESTDIR_INC_gc,
                DMA_RX_BUFFER_LEN, 
                DMA_CH_BURSTLEN_1BYTE_gc, 
                0, // Perform once
                false
                );
		} else if (dma_rx_buffer_number == 1) {
			dma_rx_buffer_number = 2;
			DMA_SetupBlock( 
                DMA_RX_Channel,
                (void *) &USART_WIFLY.DATA,
                DMA_CH_SRCRELOAD_NONE_gc, 
                DMA_CH_SRCDIR_FIXED_gc,
                dma_rx_buffer2,
                DMA_CH_DESTRELOAD_NONE_gc, 
                DMA_CH_DESTDIR_INC_gc,
                DMA_RX_BUFFER_LEN, 
                DMA_CH_BURSTLEN_1BYTE_gc, 
                0, // Perform once
                false
                );
		} else {
			dma_rx_buffer_number = 0;
			DMA_SetupBlock( 
                DMA_RX_Channel,
                (void *) &USART_WIFLY.DATA,
                DMA_CH_SRCRELOAD_NONE_gc, 
                DMA_CH_SRCDIR_FIXED_gc,
                dma_rx_buffer0,
                DMA_CH_DESTRELOAD_NONE_gc, 
                DMA_CH_DESTDIR_INC_gc,
                DMA_RX_BUFFER_LEN, 
                DMA_CH_BURSTLEN_1BYTE_gc, 
                0, // Perform once
                false
                );
			
		}
		DMA_EnableSingleShot( DMA_RX_Channel );
		// USART Trigger source, Receive complete
		DMA_SetTriggerSource( DMA_RX_Channel, DMA_CH_TRIGSRC_USART_RXC_gc); 
	
		DMA_SetIntLevel( DMA_RX_Channel, DMA_CH_TRNINTLVL_HI_gc, DMA_CH_ERRINTLVL_HI_gc );
		DMA_EnableChannel( DMA_RX_Channel );
		dma_rx_buffer_ready = true;
    } else {
        DMA.INTFLAGS = DMA_CH1ERRIF_bm;
    }
	
}


	
	
	
	
	