/*
 * dfll_trim.c
 *
 * Created: 4/13/2012 11:05:44 AM
 *  Author: Doug
 */ 

// The board.h header file defines which IO ports pheripherals like
// Switches and LEDs are connected to.
#include "board.h"

// Include header files for GCC/IAR
#include "avr_compiler.h"

// Include Clock system driver from application note AVR1003
#include "clksys_driver.h"
#include "dfll_trim.h"

void trim_to_custom_freq(void)
{
	// Enable the 32KHz internal oscillator which is used as reference for the DFLL
	CLKSYS_Enable( OSC_RC32KEN_bm );
	do {} while ( CLKSYS_IsReady( OSC_RC32KRDY_bm ) == 0 );
	

	/*  Enable automatic calibration of the internal 2MHz oscillator.
		*  We do not use the external crystal connected to the TOSC pins,
		*  but the internal 32kHz oscillator as reference.
		*/
	//CLKSYS_AutoCalibration_Enable( OSC_RC2MCREF_bm, false );

	/*  Due to an errata on ATxmega128A1 we need to enable the 32MHz internal RC oscillator
		*  and the Autocalibration for the 32MHz internal RC oscillator in order for the calibration
		*  to work correctly on the 2 MHz internal RC oscillator*/

	CLKSYS_Enable( OSC_RC32MEN_bm );
	do {} while ( CLKSYS_IsReady( OSC_RC32MRDY_bm ) == 0 );

	/* update cal registers preset values
	 * DFLLRC32M.COMP0 is not a real register
	 * The lcm of 1024 Hz and 44,100 Hz is 11289600.
	 * 
	 * Values of Clock frequency and apporpriate COMP value:
	 * 
	 * MHz      | COMP Value | Clock cycles for 44.1kHz period
	 * 11289600 | 0x2B11     | 256
	 * 22579200 | 0x5622     | 512
	 * 33868800 | 0x8133     | 768
	 * 45158400 | 0xAC44     | 1024
	 * 56448000 | 0xD755     | 1280
	 */
	
	DFLLRC32M.COMP1 = 0x33; //LSB
	DFLLRC32M.COMP2 = 0x81; //MSB
    
	//allow registers to update
	_delay_us(20);

	CLKSYS_AutoCalibration_Enable( OSC_RC32MCREF_gm, false );

	// Is this long enough to calibrate?
	_delay_ms(200);
	
	/*  Disable the automatic calibration of the internal 2MHz oscillator.
		*/
	//CLKSYS_AutoCalibration_Disable( DFLLRC2M );

	//Disable the auto calibration of the 32MHz internal RC oscillator and the 32MHz internal RC oscillator
	CLKSYS_AutoCalibration_Disable( DFLLRC32M );
	
	//switch to 32MHZ clock for main clock
	do {} while (CLKSYS_Main_ClockSource_Select( CLK_SCLKSEL_RC32M_gc ) == 0 );
	
	//disable 2MHZ clock
	CLKSYS_Disable( OSC_RC2MEN_bm );	
	
	// Disable the 32KHz internal oscillator
	CLKSYS_Disable( OSC_RC32KEN_bm );
}
