/*
 * wifly_dma.c
 *
 * Created: 4/18/2012 12:49:32 PM
 *  Author: Doug
 */ 

#include "wifly.h"
#include "dma_driver.h"
#include "board.h"
#include "usart_dma_test.h"
//#include "wifly_commands.h"

#define DMA_TX_Channel &DMA.CH0
#define DMA_RX_Channel &DMA.CH1

void wifly_send_cmd_mode(void);
void wifly_send_string(char * buffer, uint8_t len);
void wifly_test_send_string(char * buffer, uint8_t len);
void Setup_RX_Channel(char *buffer, uint8_t len);
void create_udp_connection(void);
void change_serial_to_hispeed( void );
void create_adhoc_network(void);

void initialize_wifly(void)
{
	//Setup_RX_Channel(Rx_Buf,RX_BUF_LEN);
	//enter command mode
	
	wifly_send_cmd_mode();
	
	//For Debugging
	GREEN_ON();
	if ((Rx_Buf[0] == 'C') && (Rx_Buf[1]=='M') && (Rx_Buf[2]=='D')) {BOTH_ON();}
		
	//create adhoc network
	create_adhoc_network();
	
	//enter udp state
	create_udp_connection();
	
	//save state
	wifly_send_string(wifly_save,WIFLY_SAVE_LEN);
	
	//restart
	wifly_send_string(wifly_reboot,WIFLY_REBOOT_LEN);
	
	//update our baud rate
	//change_serial_to_hispeed();
	
	//query for response
	
	//aok
}

void create_adhoc_network(void)
{
	wifly_send_string(string_1,L1);	
	wifly_send_string(string_2,L2);
	wifly_send_string(string_3,L3);
	wifly_send_string(string_4,L4);
	wifly_send_string(string_5,L5);
	wifly_send_string(string_6,L6);
}

void create_udp_connection( void ) 
{
	wifly_send_string(wifly_set_udp_mode,WIFLY_SET_UDP_MODE_LEN);
	wifly_send_string(wifly_set_rem_ip,WIFLY_SET_REM_IP_LEN);
	wifly_send_string(wifly_set_ip_rem_port,WIFLY_SET_IP_REM_PORT_LEN);
	wifly_send_string(wifly_set_ip_loc_port,WIFLY_SET_IP_LOC_PORT_LEN);
}

void wifly_send_test_ack(void)
{
	wifly_send_string(wifly_test_resp,WIFLY_TEST_RESP_LEN);
}

void change_serial_to_hispeed( void ) 
{
	wifly_send_string(wifly_set_uart_baud_hispeed_default,WIFLY_SET_UART_BAUD_HISPEED_LEN);
	
	SetupWiflyUsartHiSpeed();
}

void wifly_send_cmd_mode(void)
{
	wifly_send_string(wifly_enter_cmd_mode,WIFLY_ENTER_CMD_MODE_LEN);
}

void wifly_send_string(char * buffer, uint8_t len)
{
	
	//not thread-safe!
	
	DMA_ResetChannel(DMA_RX_Channel);
	
	DMA_SetupBlock( 
                DMA_TX_Channel,
                buffer,
                DMA_CH_SRCRELOAD_NONE_gc, 
                DMA_CH_SRCDIR_INC_gc,
                (void *) &USART_WIFLY.DATA,
                DMA_CH_DESTRELOAD_NONE_gc, 
                DMA_CH_DESTDIR_FIXED_gc,
                len, 
                DMA_CH_BURSTLEN_1BYTE_gc, 
                0, // Perform once
                false
                );
				
    DMA_EnableSingleShot( DMA_TX_Channel);
    // USART Trigger source, Data Register Empty, 0x4C
    DMA_SetTriggerSource( DMA_TX_Channel, DMA_CH_TRIGSRC_USART_DRE_gc);
	
	DMA_SetupBlock( 
                DMA_RX_Channel,
                (void *) &USART_WIFLY.DATA,
                DMA_CH_SRCRELOAD_NONE_gc, 
                DMA_CH_SRCDIR_FIXED_gc,
                Rx_Buf,
                DMA_CH_DESTRELOAD_NONE_gc, 
                DMA_CH_DESTDIR_INC_gc,
                len, 
                DMA_CH_BURSTLEN_1BYTE_gc, 
                0, // Perform once
                false
                );
        
    DMA_EnableSingleShot( DMA_RX_Channel );
    // USART Trigger source, Receive complete
    DMA_SetTriggerSource( DMA_RX_Channel, DMA_CH_TRIGSRC_USART_RXC_gc);

	// The receiving DMA channel will wait for characters 
    // and write them to Rx_Buf
    DMA_EnableChannel( DMA_RX_Channel );
	
	// Start sending the data bits from Tx_Buf on the
    // transmit dma channel.
    // Note that DRE is considered empty at this point,
    // thus triggering the initial transfer.
    DMA_EnableChannel( DMA_TX_Channel );
	
	// wait for response
	while ( !DMA_ReturnStatus_non_blocking( DMA_RX_Channel ))
	{
		nop();
	}
}

void wifly_test_send_string(char * buffer, uint8_t len)
{
	uint8_t i=0;
	DMA_ResetChannel(DMA_RX_Channel);
	
	DMA_SetupBlock( 
                DMA_TX_Channel,
                buffer,
                DMA_CH_SRCRELOAD_BLOCK_gc, 
                DMA_CH_SRCDIR_INC_gc,
                (void *) &USART_WIFLY.DATA,
                DMA_CH_DESTRELOAD_NONE_gc, 
                DMA_CH_DESTDIR_FIXED_gc,
                len, 
                DMA_CH_BURSTLEN_1BYTE_gc, 
                0, // Perform once
                false
                );
				
    DMA_EnableSingleShot( DMA_TX_Channel);
    // USART Trigger source, Data Register Empty, 0x4C
    DMA_SetTriggerSource( DMA_TX_Channel, DMA_CH_TRIGSRC_USART_DRE_gc);
	
	DMA_SetupBlock( 
                DMA_RX_Channel,
                (void *) &USART_WIFLY.DATA,
                DMA_CH_SRCRELOAD_NONE_gc, 
                DMA_CH_SRCDIR_FIXED_gc,
                Rx_Buf,
                DMA_CH_DESTRELOAD_BLOCK_gc, 
                DMA_CH_DESTDIR_INC_gc,
                len, 
                DMA_CH_BURSTLEN_1BYTE_gc, 
                0, // Perform once
                false
                );
        
    DMA_EnableSingleShot( DMA_RX_Channel );
    // USART Trigger source, Receive complete
    DMA_SetTriggerSource( DMA_RX_Channel, DMA_CH_TRIGSRC_USART_RXC_gc); 

	// The receiving DMA channel will wait for characters 
    // and write them to Rx_Buf
    DMA_EnableChannel( DMA_RX_Channel );
	
	// Start sending the data bits from Tx_Buf on the
    // transmit dma channel.
    // Note that DRE is considered empty at this point,
    // thus triggering the initial transfer.
    DMA_EnableChannel( DMA_TX_Channel );
	
	// wait for response
	while ( !DMA_ReturnStatus_non_blocking( DMA_RX_Channel ))
	{
		nop();
	}
	
	// Verify the received characters
    for(i=0; i<len; i++)
    {
        if(buffer[i] != Rx_Buf[i])
        {
            // Something went wrong..
            //while (1)
			{
				if (LEDPORT.OUTCLR != 0x7F){
					// Light means failure!
					if (i==0){LEDPORT.OUT = 0x00;}
					if (i==1){LEDPORT.OUT = 0x01;}
					if (i>=2){LEDPORT.OUT = 0x02;}
				}				
			}				
        }
    }
}